/*
Map for Gyroscope
using Google Maps API
https://www.antradar.com/docs-gsmap-js
version: 2024-9-30-1
*/

gsmap_getmap=function(id, lat, lng, zoom, callback, myopts, mapgroup){
if (!self.gsmap_apikey||gsmap_apikey==''){
	console.log("missing gsmap_apikey");
	return;
}
var apilink='https://maps.googleapis.com/maps/api/js?key='+gsmap_apikey+'&region=US';
xajxjs('google.maps',apilink,function(){
	if (!document.gsmaplot) {//init registry
		document.gsmaplot=[];
		document.gsmappark=document.createElement('div');
	}

	//find an available slot
	var mapx=null;

	for (var i=0;i<document.gsmaplot.length;i++){
		var slot=document.gsmaplot[i];
		if (slot.free||slot.mapx.mapview.id==id){
			if ((mapgroup!=null||slot.mapgroup!=null)&&slot.mapgroup!=mapgroup){
				console.log('cannot use free map slot #'+i+' because '+(slot.mapgroup??'it')+' is not part of the '+(mapgroup||'*noname*')+' group');
				continue;	
			}
			
			console.log('found free map slot #'+i, id);
			slot.free=false;
			mapx=slot.mapx;
			break;
		}
	}//mapx

	if (mapx){
		mapx.mapobj.setCenter(new google.maps.LatLng(lat,lng));
		mapx.mapobj.setZoom(zoom);

	}

	if (!mapx){
		console.log('creating a new map');
		
		var center=new google.maps.LatLng(lat,lng);
		var opts = {
		zoom: zoom,
		mapTypeControl:false,
		panControl:false,
		zoomControl: false,
		streetViewControl: false,
		center: center,
		mapTypeId: google.maps.MapTypeId.ROADMAP
		};
		
		if (myopts!=null) {
			opts=myopts;
			opts.center=center;
		}

		mapx=document.createElement('div');
		mapx.style.width='100%';
		mapx.style.height='100%';
		mapx.mapobj=new google.maps.Map(mapx,opts);

		var slot={free:false, mapx: mapx, mapgroup:mapgroup}
		document.gsmaplot.push(slot);
		mapx.pslot=slot;
	} else {
		if (myopts!=null) mapx.mapobj.setOptions(myopts);	
	}

	var mapview=gid(id);
	mapx.mapview=mapview;
	mapview.appendChild(mapx);
	mapview.mapx=mapx;

	if (callback) callback();
});
}


gsmap_freemap=function(id){
	var mapview=gid(id);
	if (!mapview) return;
	if (!mapview.mapx) return;
	
	gsmap_clearmarkers(id);
	gsmap_clearlisteners(id);
	
	mapview.mapx.markers=[];
	mapview.mapx.pslot.free=true;
	mapview.mapx.parentNode.removeChild(mapview.mapx);
	
	document.gsmappark.appendChild(mapview.mapx);
	mapview.mapx=null;
}

/*
each marker carries its own settings
myopt overrides every marker in the array
*/
gsmap_addmarkers=function(id,markers,myopts){
	var mapview=gid(id);
	if (!mapview) return;
	if (!mapview.mapx) return;

	if (!mapview.mapx.markers) mapview.mapx.markers=[];

	var refs=[];

	for (var i=0;i<markers.length;i++){
		var m=markers[i];
		var opts={
			position:new google.maps.LatLng(m.lat,m.lng),
			map:mapview.mapx.mapobj,
			icon:m.icon
		};
		
		if (myopts!=null){
			for (k in myopts){
				if (opts[k]==null) opts[k]=myopts[k];	
			}	
		}
		
		var marker=new google.maps.Marker(opts);
		refs.push(marker);
		mapview.mapx.markers.push(marker);
	}

	return refs;
}

gsmap_clearmarkers=function(id){
	var mapview=gid(id);
	if (!mapview) return;
	if (!mapview.mapx) return;
	
	if (mapview.mapx.markers){
		for (var i=0;i<mapview.mapx.markers.length;i++){
			mapview.mapx.markers[i].setMap(null);
		}
	}

	mapview.mapx.markers=[];	
}

gsmap_addlistener=function(id,event_type,func){
	var mapview=gid(id);
	if (!mapview) return;
	if (!mapview.mapx) return;

	if (!mapview.mapx.listeners) mapview.mapx.listeners=[];
	
	var listener=mapview.mapx.mapobj.addListener(event_type,func);
	mapview.mapx.listeners.push(listener);
}

gsmap_clearlisteners=function(id){
	var mapview=gid(id);
	if (!mapview) return;
	if (!mapview.mapx) return;
	
	if (mapview.mapx.listeners){
		for (var i=0;i<mapview.mapx.listeners.length;i++){
			google.maps.event.removeListener(mapview.mapx.listeners[i]);	
		}	
	}
	
	mapview.mapx.listeners=[];
	
}

gsmap_getpano=function(id, lat, lng, heading, pitch, callback, myopts, panogroup){
if (!self.gsmap_apikey||gsmap_apikey==''){
	console.log("missing gsmap_apikey");
	return;
}
var apilink='https://maps.googleapis.com/maps/api/js?key='+gsmap_apikey+'&region=US';
xajxjs('google.maps',apilink,function(){
	if (!document.gspanolot) {//init registry
		document.gspanolot=[];
		document.gspanopark=document.createElement('div');
	}

	//find an available slot
	var mapx=null;

	for (var i=0;i<document.gspanolot.length;i++){
		var slot=document.gspanolot[i];
		if (slot.free||slot.mapx.panoview.id==id){
			if ((panogroup!=null||slot.panogroup!=null)&&slot.panogroup!=panogroup){
				console.log('cannot use free pano slot #'+i+' because '+(slot.panogroup??'it')+' is not part of the '+(panogroup||'*noname*')+' group');
				continue;	
			}
			
			console.log('found free pano slot #'+i, id);
			slot.free=false;
			mapx=slot.mapx;
			break;
		}
	}//mapx


	if (mapx){
		mapx.mapobj.setPosition(new google.maps.LatLng(lat,lng));
		mapx.mapobj.setPov({heading:heading,pitch:pitch});

	}


	if (!mapx){
		console.log('creating a new pano');
		
		var center=new google.maps.LatLng(lat,lng);
		var opts = {
			position:center,
			pov:{
				heading:heading,
				pitch:pitch
			}
		};
		
		if (myopts!=null) {
			opts=myopts;
			opts.center=center;
		}

		mapx=document.createElement('div');
		mapx.style.width='100%';
		mapx.style.height='100%';
		mapx.mapobj=new google.maps.StreetViewPanorama(mapx,opts);

		var slot={free:false, mapx: mapx, panogroup:panogroup}
		document.gspanolot.push(slot);
		mapx.pslot=slot;
	} else {
		if (myopts!=null) mapx.mapobj.setOptions(myopts);	
	}

	var panoview=gid(id);
	mapx.panoview=panoview;
	panoview.appendChild(mapx);
	panoview.mapx=mapx;

	if (callback) callback();
});
}

gsmap_freepano=function(id){
	var panoview=gid(id);
	if (!panoview) return;
	if (!panoview.mapx) return;
	
	//gsmap_clearmarkers(id);
	gsmap_clearlisteners(id);
	
	panoview.mapx.markers=[];
	panoview.mapx.pslot.free=true;
	panoview.mapx.parentNode.removeChild(panoview.mapx);
	
	document.gspanopark.appendChild(panoview.mapx);
	panoview.mapx=null;
}

